@@ -1,8 +1,4 @@
 error_chain::error_chain! {
-    foreign_links {
-        Regex(::regex::Error);
-    }
-
     errors {
         MessageNotFound(msg: String, folders: String) {
             description("message not found in provided directories")
